#import sys
#sys.path.append('../')

import unittest
import numpy as np
from scipy.stats import norm
from numpy.testing import assert_equal, assert_almost_equal, assert_
from ..parameter.gauss_class import Gauss

test_norm1 = norm(loc=5.0, scale=1.0)
test_norm2 = norm(loc=6.0, scale=0.5)
loc_scale = np.atleast_2d([(5.0, 1.0), (6.0, 0.5)])
test_gauss = Gauss(loc_scale, 1.0, 10.0)

class TestGaussClass(unittest.TestCase):
    """
    Tests for the Gauss class.
    """

    def test_init_locscale(self):
        assert_equal(test_gauss.data,loc_scale)

    def test_init_lb(self):
        assert_equal(test_gauss.lb, 1.0)

    def test_init_un(self):
        assert_equal(test_gauss.ub, 10.0)

    def test_pdf(self):
        assert_almost_equal(test_gauss.pdf(1.0), (0.5 * test_norm1.pdf(1.0)) + (0.5 * test_norm2.pdf(1.0)))

    def test_logpdf(self):
        assert_almost_equal(test_gauss.logpdf(1.0), np.log((0.5 * test_norm1.pdf(1.0)) + (0.5 * test_norm2.pdf(1.0))))

    def test_cdf(self):
        assert_almost_equal(test_gauss.cdf(1.0), (0.5 * test_norm1.cdf(1.0)) + (0.5 * test_norm2.cdf(1.0)))

    def test_ppf(self):
        loc_scale2 = np.atleast_2d([(5.0, 1.0)])
        test_gauss2 = Gauss(loc_scale2, 1.0, 10.0)
        assert_almost_equal(test_gauss2.ppf(0.5), test_norm1.ppf(0.5))

    def test_ppf_root(self):
        loc_scale2 = np.atleast_2d([(5.0, 1.0)])
        test_gauss2 = Gauss(loc_scale2, 1.0, 10.0)
        assert_almost_equal(test_gauss2._ppf_root(5.0,1.0), test_norm1.cdf(5.0) - 1.0)

    def test_rvs(self):
        loc_scale2 = np.atleast_2d([(5.0, 1.0)])
        test_gauss2 = Gauss(loc_scale2, 1.0, 10.0)
        x = test_gauss2.rvs(10)
        assert_(len(x) == 10)




import numpy as np
from scipy.optimize import root_scalar
from scipy.stats import norm


class Gauss:
    """
    Creates an informed prior probability distribution through a weighted sum of Gaussian distributions
    for literature values of a parameter. Can return statistical properties of this distribution and has
    an rvs method to return random variates from the distribution.
    """

    def __init__(self, loc_scale, lb, ub):
        # An array of the loc and scale values used to create each Gaussian.
        self.data = np.atleast_2d(loc_scale)
        # The xrange to use for the distribution.
        self.lb = lb
        self.ub = ub
        self.xrange = np.linspace(self.lb, self.ub, 100)
        # The weight each Gaussian has in the sum: currently equally weighted.
        self.weight = 1.0 / len(self.data)
        self._pdf_ = []
        # Create arrays from the data
        self.distros = []
        for d in self.data:
            self.distros.append(norm(loc=d[0], scale=d[1]))

    def pdf(self, x):
        """
        Loops over the values in the data array to find the gaussian for each value. The Gaussians
        are then added, taking the weight into consideration. The value of the pdf at the x value
        given is then returned.
        """
        x = np.atleast_1d(x)
        _pdf = np.zeros_like(x)
        for i, d in enumerate(self.distros):
            for j, k in enumerate(x):
                if k < self.lb:
                    _pdf[j] = 0
                elif k >= self.ub:
                    _pdf[j] = 0
                else:
                    _pdf[j] += d.pdf(x[j]) * self.weight

        return _pdf

    def logpdf(self, x):
        """
        Returns the log of the pdf at x.
        """
        return np.log(self.pdf(x))

    def cdf(self, x):
        """
        Returns the cumulative distribution function at x.
        """
        _cdf = np.zeros_like(x)
        for i, d in enumerate(self.distros):
            _cdf += d.cdf(x) * self.weight
        return _cdf

    def ppf(self, x):
        """
        Returns the percent point function (quantile function / inverse cdf) at x.
        """
        brack_min = np.mean(self.data[:, 0]) - np.mean(self.data[:, 0]) * 10
        brack_max = np.mean(self.data[:, 0]) + np.mean(self.data[:, 0]) * 10
        x = np.atleast_1d(x)
        _ppf = np.zeros_like(x)
        for i, v in enumerate(x):
            if v < 1e-7:
                _ppf[i] = self.lb
            elif v > (1.0 - 1e-7):
                _ppf[i] = self.ub
            else:
                _ppf[i] = root_scalar(self._ppf_root,
                                      bracket=[brack_min, brack_max],
                                      args=[v]).root
        if len(_ppf) == 1:
            return _ppf[0]
        return _ppf

    def _ppf_root(self, y, x):
        return self.cdf(y) - x

    def rvs(self, n):
        """
        Return n number of random numbers from the pdf.
        """
        gauss = self.pdf(self.xrange)
        prob = gauss / sum(gauss)  # normalise the pdf so the values sum to 1
        _rvs = np.random.choice(self.xrange, p=prob, size=n)
        return _rvs




import unittest
import numpy as np
from numpy.testing import assert_almost_equal
from ..parameter.gauss_class import Gauss
from ..parameter import dh as dh

class TestDh(unittest.TestCase):

    def test_Gauss(self):
        a = dh.Gauss('DMPC')
        loc_scale = np.atleast_2d([(8.0, 1.0), (9.5, 1.6), (8.4, 1.1), (9.0,2.0)])
        b = Gauss(loc_scale, 4.0, 22.0)
        assert_almost_equal(a.pdf(0.5),b.pdf(0.5))

    def test_uniform(self):
        b = dh.uniform('DMPC')
        assert_almost_equal(b,np.array([[4.0,22.0]]))












import unittest
import numpy as np
from numpy.testing import assert_almost_equal
from ..parameter.gauss_class import Gauss
from ..parameter import vh as vh


class TestVh(unittest.TestCase):

    def test_Gauss(self):
        a = vh.Gauss('DMPC')
        loc_scale = np.atleast_2d([(320.9,20.1), (339.5,14.5), (319.0,6.0)])
        b = Gauss(loc_scale, 250.0, 450.0)
        assert_almost_equal(a.pdf(0.5), b.pdf(0.5))

    def test_uniform(self):
        b = vh.uniform('DMPC')
        assert_almost_equal(b, np.array([[250.0, 450.0]]))


from .util import findGauss, findUniform
import matplotlib.pyplot as plt
import numpy as np


def Gauss(name):
    """"
    Calls the findGauss function from util.py to return the Gauss prior for the given molecule.
    Input:      name of molecule
    Output:     Gauss prior object containing pdf, logpdf, cdf, ppf and rvs methods
    """
    prior = findGauss(name, 'v_t')
    return prior


def uniform(name):
    """
    Calls the findUniform function from util.py to return the uniform bounds for the given molecule.
    Input:      name of molecule
    Output:     array of length [2] with the upper and lower bounds for the uniform prior
    """
    prior = findUniform(name, 'v_t')
    return prior


def plotGauss(name): # pragma: no cover
    """
    Plots the prior probability distribution for the given molecule.
    Input:      name of molecule, type of prior (should either be 'Gauss' or 'uniform')
    Output:     matplotlib.pyplot graph of the given prior
    """
    # set the xrange, upper bound and lower bound for the prior
    xrange = uniform(name)
    lb = xrange[0,0]
    ub = xrange[0,1]
    xrange = np.linspace(lb, ub, 100)

    # plot the Gauss prior
    prior = Gauss(name)
    plt.xlabel('Tail Volume [Å]')
    plt.ylabel('pdf')
    plt.title(name)
    plt.plot(xrange, prior.pdf(xrange))
    plt.show()

def plotUniform(name): # pragma: no cover

    xrange = uniform(name)
    lb = xrange[0,0]
    ub = xrange[0,1]
    xrange = np.linspace(0.5 * lb, 1.3 * ub, 100)
    y = np.zeros_like(xrange)

    # plot the uniform prior
    for i, j in enumerate(xrange):
        if (lb <= j <= ub):
            y[i] = 1.0
        else:
            y[i] = 0.0
    plt.xlabel('Tail volume [Å]')
    plt.ylabel('pdf')
    plt.title(name)
    plt.plot(xrange, y)
    plt.show()




import unittest
import numpy as np
from numpy.testing import assert_almost_equal
from ..parameter.gauss_class import Gauss
from ..parameter import vt as vt


class TestVt(unittest.TestCase):

    def test_Gauss(self):
        a = vt.Gauss('DMPC')
        loc_scale = np.atleast_2d([(851.5,5.0), (815.9,15.5)])
        b = Gauss(loc_scale, 700.0, 1000.0)
        assert_almost_equal(a.pdf(0.5), b.pdf(0.5))

    def test_uniform(self):
        b = vt.uniform('DMPC')
        assert_almost_equal(b, np.array([[700.0, 1000.0]]))


#import sys
#sys.path.append('../')

import unittest
import numpy as np
from numpy.testing import assert_equal, assert_almost_equal
from ..parameter.gauss_class import Gauss
from ..parameter.util import findGauss, findUniform

loc_scale = np.atleast_2d([(320.9, 20.1), (339.5, 14.5),(319.0,6.0)])
test_gauss = Gauss(loc_scale, 250.0, 450.0)

class TestUtil(unittest.TestCase):

    def test_findGauss(self):
        a = findGauss('DMPC','v_h')
        assert_almost_equal(a.pdf(0.5),test_gauss.pdf(0.5))

    def test_findUniform(self):
        b = findUniform('DMPC','v_h')
        assert_almost_equal(b,np.array([[250.0,450.0]]))


from .gauss_class import Gauss
import pandas as pd
import bson



def findGauss(name, parameter):
    """
    Opens the database to extract the data, inserts it into Gauss class and returns a Gauss prior object.
    Input:      name of molecule, parameter to find
    Output:    Gauss prior object containing pdf, logpdf, cdf, ppf and rvs methods
    """
    # Open the bson_file to get the Gaussian data
    bson_file = open('../avant/db/Gaussian.bson', 'rb')
    b = bson.decode_all(bson_file.read())
    # Turn the data into a pandas dataframe so it can be filtered
    c = pd.DataFrame(list(b))
    # Filter the data by the correct name and parameter
    filtered = c[(c['name'] == name) & (c['parameter'] == parameter)]
    # Return only the loc and scale columns
    filtered2 = filtered[['loc', 'scale']]
    # Convert to numpy array
    data_arr = filtered2.to_numpy(dtype=float)

    # get the values for the lower and upper bound of the distribution
    bounds = findUniform(name,parameter)
    lb = bounds[0,0]
    ub = bounds[0,1]
    # insert the data into Gauss class to get the prior probability object
    prior_object = Gauss(data_arr,lb,ub)
    print(data_arr,lb,ub)

    return prior_object


def findUniform(name, parameter):
    """
    Opens the database to extract the data and returns an array of bounds for the uniform prior probability.
    Input:      name of molecule, parameter of interest
    Output:     array of length [2] with the upper and lower bounds for the uniform prior
    """

    # Open the bson_file to get the Uniform data
    bson_file = open('../avant/db/uniform.bson', 'rb')
    b = bson.decode_all(bson_file.read())
    # Turn the data into a pandas dataframe so it can be filtered
    c = pd.DataFrame(list(b))
    # Filter the data by the correct name and parameter
    filtered = c[(c['name'] == name) & (c['parameter'] == parameter)]
    # Return only the upper and lower bound columns
    filtered2 = filtered[['lower bound', 'upper bound']]
    # Convert to numpy array
    bounds = filtered2.to_numpy(dtype=float)

    return bounds





import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="avant",
    version = "0.0.1",
    author="Nicola Farmer",
    author_email="nkf679@gmail.com",
    description="A Python package to create informed prior probability distributions for reflectometry analysis",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/nf679/avant",
    project_urls={
        "docs": "https://avant.readthedocs.io/en/latest/",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
    )




import unittest
import numpy as np
from numpy.testing import assert_almost_equal
from ..parameter.gauss_class import Gauss
from ..parameter import dt as dt


class TestDt(unittest.TestCase):

    def test_Gauss(self):
        a = dt.Gauss('DMPC')
        loc_scale = np.atleast_2d([(27.96,0.3), (29.0,2.0), (28.0,0.6)])
        b = Gauss(loc_scale, 10.0, 50.0)
        assert_almost_equal(a.pdf(0.5), b.pdf(0.5))

    def test_uniform(self):
        b = dt.uniform('DMPC')
        assert_almost_equal(b, np.array([[10.0, 50.0]]))




import unittest
import numpy as np
from numpy.testing import assert_almost_equal
from ..parameter.gauss_class import Gauss
from ..parameter import roughness as r


class TestRoughness(unittest.TestCase):

    def test_Gauss(self):
        a = r.Gauss('DMPC')
        loc_scale = np.atleast_2d([(4.42,0.16), (3.9,0.6), (2.5,0.5),(3.02,0.08),(5.0,2.0),(3.3,0.4)])
        b = Gauss(loc_scale, 0.0, 10.0)
        assert_almost_equal(a.pdf(0.5), b.pdf(0.5))

    def test_uniform(self):
        b = r.uniform('DMPC')
        assert_almost_equal(b, np.array([[0.0, 10.0]]))